import { NostrChannel } from './channel.js';
import { EventEmitter } from './emitter.js';
import { NostrSocket } from './socket.js';
import { NostrStore } from './store.js';
const ROOM_DEFAULTS = () => {
    return {
        debug: false,
        echo: false,
        verbose: false
    };
};
export class NostrRoom extends EventEmitter {
    static { this.list = NostrStore.list; }
    constructor(secret, signer, options) {
        const opt = { ...ROOM_DEFAULTS(), ...options };
        const socket = new NostrSocket(opt);
        super();
        this._opt = opt;
        this._socket = socket;
        this._store = new NostrStore(secret, signer, { ...opt, socket });
        this._sub = new NostrChannel(secret, signer, { ...opt, socket });
        this._init = false;
        this._store.once('ready', () => void this._initialize());
        this._store.on('update', () => void this.emit('update', this));
        this._sub.once('ready', () => void this._initialize());
        this._sub.on('msg', (msg) => void this.emit('msg', msg));
    }
    get data() {
        return this._store.data;
    }
    get store() {
        return this._store.store;
    }
    _initialize() {
        if (this._store.ready && this._sub.ready) {
            this._init = true;
            this.emit('ready', this);
        }
    }
    on_topic(topic, fn) {
        return this._sub.on_topic(topic, fn);
    }
    async connect(address) {
        this._store.fetch();
        this._sub.fetch();
        this._socket.connect(address);
        return this;
    }
    delete() {
        return this._store.delete();
    }
    async init(address, data) {
        this._store.update(data);
        this._sub.fetch();
        this._socket.connect(address);
        return this;
    }
    send(subject, body) {
        return this._sub.send(subject, body);
    }
    update(data) {
        return this._store.update(data);
    }
}
